==> com.koki-ibukuro.arkitstream-0.2.2.tgz: package/Runtime/Internal/ARKitRemotePacket.cs <==
﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using UnityEngine;
using Unity.Mathematics;

namespace ARKitStream.Internal
{
    [Serializable]
    public class ARKitRemotePacket
    {
        [Serializable]
        public struct CameraFrameEvent : IEquatable<CameraFrameEvent>
        {
            // public ARLightEstimationData lightEstimationl;
            public long timestampNs;
            public float4x4 projectionMatrix;
            public float4x4 displayMatrix;
            // public double? exposureDuration;
            // public float? exposureOffset;

            public bool Equals(CameraFrameEvent o)
            {
                return timestampNs.Equals(o.timestampNs)
                    && projectionMatrix.Equals(o.projectionMatrix)
                    && displayMatrix.Equals(o.displayMatrix);
            }

            public override string ToString()
            {
                return $"[time: {timestampNs}, projection: {projectionMatrix}, display: {displayMatrix}]";
            }
        }

        [Serializable]
        public class FaceMesh
        {
            public TrackableId id;
            public byte[] vertices; // NativeArray<Vector3>
            public byte[] normals; // NativeArray<Vector3>
            public byte[] indices; // NativeArray<int>
            public byte[] uvs; // NativeArray<Vector2>
            public byte[] coefficients;

            public override string ToString()
            {
                return $"Mesh {id} verts: {vertices.Length} norms: {normals.Length} indices: {indices.Length} uvs: {uvs.Length}";
            }
        }

        [Serializable]
        public class FaceInfo
        {
            public XRFace[] added;
            public XRFace[] updated;
            public TrackableId[] removed;
            public FaceMesh[] meshes;

            public override string ToString()
            {
                var sb = new System.Text.StringBuilder();
                sb.AppendLine("FaceInfo");
                foreach (var f in added)
                {
                    sb.AppendLine($"ADD: {f}");
                }
                foreach (var f in updated)
                {
                    sb.AppendLine($"UPDATE: {f}");
                }
                foreach (var f in removed)
                {
                    sb.AppendLine($"REMOVE: {f}");
                }
                foreach (var m in meshes)
                {
                    sb.AppendLine($"MESHED: {m}");
                }
                return sb.ToString();
            }
        }

        [Serializable]
        public class PlaneInfo
        {
            public BoundedPlane[] added;
            public BoundedPlane[] updated;
            public TrackableId[] removed;
            public Dictionary<TrackableId, byte[]> meshes;

            public override string ToString()
            {
                var sb = new System.Text.StringBuilder();
                sb.AppendLine("PlaneInfo");
                foreach (var f in added)
                {
                    sb.AppendLine($"ADD: {f}");
                }
                foreach (var f in updated)
                {
                    sb.AppendLine($"UPDATE: {f}");
                }
                foreach (var f in removed)
                {
                    sb.AppendLine($"REMOVE: {f}");
                }
                foreach (var m in meshes)
                {
                    sb.AppendLine($"MESHED: {m}");
                }
                return sb.ToString();
            }
        }

        public CameraFrameEvent cameraFrame;
        public FaceInfo face;
        public Pose trackedPose;
        public PlaneInfo plane;

        static readonly BinaryFormatter formatter = new BinaryFormatter();
        static readonly MemoryStream stream = new MemoryStream();
        static byte[] buffer = new byte[0];

        public byte[] Serialize()
        {
            // BinaryFormatter is slow. shold make custom serialization?
            lock (stream)
            {
                stream.Position = 0;
                formatter.Serialize(stream, this);
                int length = (int)stream.Position;
                if (buffer.Length != length)
                {
                    buffer = new byte[length];
                }
                stream.Position = 0;
                stream.Read(buffer, 0, length);
            }
            return buffer;
        }

        public static ARKitRemotePacket Deserialize(byte[] data)
        {
            lock (stream)
            {
                stream.Position = 0;
                stream.Write(data, 0, data.Length);
                stream.Position = 0;
                return formatter.Deserialize(stream) as ARKitRemotePacket;
            }
        }
    }

}

==> com.koki-ibukuro.arkitstream-0.4.2.tgz: package/Runtime/Internal/ARKitRemotePacket.cs <==
﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using UnityEngine;
using Unity.Mathematics;

namespace ARKitStream.Internal
{
    [Serializable]
    public partial class ARKitRemotePacket
    {
        public CameraFrameEvent cameraFrame;
        public FaceInfo face;
        public Pose trackedPose;
        public PlaneInfo plane;
        public HumanBodyInfo humanBody;

        static readonly BinaryFormatter formatter = new BinaryFormatter();
        static readonly MemoryStream stream = new MemoryStream();
        static byte[] buffer = new byte[0];

        public byte[] Serialize()
        {
            // BinaryFormatter is slow. shold make custom serialization?
            lock (stream)
            {
                stream.Position = 0;
                formatter.Serialize(stream, this);
                int length = (int)stream.Position;
                if (buffer.Length != length)
                {
                    buffer = new byte[length];
                }
                stream.Position = 0;
                stream.Read(buffer, 0, length);
            }
            return buffer;
        }

        public static ARKitRemotePacket Deserialize(byte[] data)
        {
            lock (stream)
            {
                stream.Position = 0;
                stream.Write(data, 0, data.Length);
                stream.Position = 0;
                return formatter.Deserialize(stream) as ARKitRemotePacket;
            }
        }
    }

}
